ClassicPremadeFrame = CreateFrame("Frame");
local QUEUE_TIME = 0;
local HAS_SENT_READY = false;
local READY_CHECK = 0;
local MAX_DELAY = 0;
local NOT_READY = {};

local READY_CHECK_COOLDOWN = 0;
local QUEUE_COOLDOWN = 0;

local SOUND_PLAYED = false;

function ClassicPremade_GetLag()
  local down, up, lagHome, lagWorld = GetNetStats();
  return lagHome / 1000;
end

function ClassicPremade_HandleQueue()
  if(not SOUND_PLAYED and QUEUE_TIME - GetTime() < 1) then
    SOUND_PLAYED = true;
    PlaySound(8959);
  end
  if(QUEUE_TIME < GetTime()) then
    ClassicPremadeFrame:SetScript("OnUpdate", nil);
    BattlefieldFrameJoinButton:SetEnabled(true);
    QUEUE_TIME = 0;
    ClassicPremadeQueueButton:SetEnabled(true);
  end
end

function ClassicPremade_IsReady()
  return strfind(BattlefieldFrameZoneDescription:GetText(), "Alterac") ~= nil and BattlefieldFrame:IsVisible();
end

function ClassicPremade_ReadyCheck()
  local group = ClassicPremade_GroupType();
  NOT_READY = {};
  for i = 1, GetNumGroupMembers() do
    local name = UnitName(group .. i);
    if(name ~= nil) then
      NOT_READY[name] = true;
    end
  end
end

function ClassicPremade_PerformReadyCheck(sender)
  if(not ClassicPremade_PlayerIsLeader(sender) or READY_CHECK > 0) then
    return;
  end
  print("[ClassicPremade] Ready check started.");
  PlaySound(8960);
  READY_CHECK = GetTime();
  ClassicPremade_ReadyCheck();
  ClassicPremadeFrame:SetScript("OnUpdate", ClassicPremade_HandleReadyCheck);
end

function ClassicPremade_HandleReadyCheck()
  if(not HAS_SENT_READY) then
    if(ClassicPremade_IsReady()) then
      C_ChatInfo.SendAddonMessage("CPREM-R", "-", strupper(ClassicPremade_GroupType()));
      HAS_SENT_READY = true;
    end
  end
  if(READY_CHECK + 7 < GetTime()) then
    ClassicPremadeFrame:SetScript("OnUpdate", nil);
    ClassicPremadeReadyButton:SetEnabled(true);
    READY_CHECK = 0;
    local notReadyPlayers = "";
    for k, v in pairs(NOT_READY) do
      if(notReadyPlayers ~= "") then
        notReadyPlayers = notReadyPlayers .. ", ";
      end
      notReadyPlayers = notReadyPlayers .. k;
    end
    notReadyPlayers = notReadyPlayers .. ".";
    if(notReadyPlayers == ".") then
      print("[ClassicPremade] Everyone is ready.");
    else
      print("[ClassicPremade] Not ready:");
      print(notReadyPlayers);
    end
  end
end

function ClassicPremade_Queue(sender, delay)
  if(not ClassicPremade_PlayerIsLeader(sender)) then
    return;
  end
  print("[ClassicPremade] Queue starting in " .. delay .. "s.");
  SOUND_PLAYED = false;
  BattlefieldFrameJoinButton:SetEnabled(false);
  QUEUE_TIME = GetTime() + delay - ClassicPremade_GetLag() * 2;
  ClassicPremadeFrame:SetScript("OnUpdate", ClassicPremade_HandleQueue);
end

function ClassicPremade_PlayerIsLeader(sender)
  if(UnitIsGroupLeader("player"))  then
    return UnitName("player") == sender;
  end
  local group = ClassicPremade_GroupType();
  for i = 1, GetNumGroupMembers() do
    if(UnitIsGroupLeader(group .. i)) then
      return UnitName(group ..i) == sender;
    end
  end
  return false;
end

function ClassicPremade_GroupType()
  return IsInRaid() and "raid" or "party";
end

function ClassicPremade_HandleMessage(self, event, arg1, arg2, arg3, arg4, ...)
  arg4, _ = strsplit("-", arg4);
  if(arg1 == "CPREM-RC") then
    ClassicPremade_PerformReadyCheck(arg4);
  elseif(arg1 == "CPREM-R") then
    if(READY_CHECK > 0) then
      NOT_READY[arg4] = nil;
      MAX_DELAY = GetTime() - READY_CHECK;
      if(ClassicPremade_PlayersAreReady()) then
        READY_CHECK = 0;
      end
    end
  elseif(arg1 == "CPREM-Q") then
    arg2 = tonumber(arg2);
    if(arg2 ~= nil) then
      if(arg2 < 15) then
        ClassicPremade_Queue(arg4, arg2);
      end
    end
  end
end

function ClassicPremade_PlayersAreReady()
  for k,v in pairs(NOT_READY) do
    if(v ~= nil) then
      return false;
    end
  end
  return true;
end

ClassicPremadeFrame:SetScript("OnEvent", ClassicPremade_HandleMessage);
ClassicPremadeFrame:RegisterEvent("CHAT_MSG_ADDON");

C_ChatInfo.RegisterAddonMessagePrefix("CPREM-RC");
C_ChatInfo.RegisterAddonMessagePrefix("CPREM-R");
C_ChatInfo.RegisterAddonMessagePrefix("CPREM-Q");

local b = CreateFrame("Button", "ClassicPremadeQueueButton", BattlefieldFrame, "UIPanelButtonTemplate")
b:SetSize(100, 24);
b:SetText("Group Queue");
b:SetPoint("BOTTOMRIGHT", BattlefieldFrame, "BOTTOMRIGHT", -35, 50);
b:SetScript("OnClick", function()
  if(ClassicPremade_PlayerIsLeader(UnitName("player")) and QUEUE_COOLDOWN < GetTime()) then
    QUEUE_COOLDOWN = GetTime() + 15;
    local delay = MAX_DELAY + ClassicPremade_GetLag();
    if(delay < 3) then
      delay = 3;
    end
    C_ChatInfo.SendAddonMessage("CPREM-Q", delay, strupper(ClassicPremade_GroupType()));
    ClassicPremadeQueueButton:SetEnabled(false);
  end
end)

local b = CreateFrame("Button", "ClassicPremadeReadyButton", BattlefieldFrame, "UIPanelButtonTemplate")
b:SetSize(100, 24);
b:SetText("Ready Check");
b:SetPoint("BOTTOMRIGHT", BattlefieldFrame, "BOTTOMRIGHT", -35 - 100, 50);
b:SetScript("OnClick", function()
  if(ClassicPremade_PlayerIsLeader(UnitName("player")) and READY_CHECK_COOLDOWN < GetTime()) then
    READY_CHECK_COOLDOWN = GetTime() + 8;
    C_ChatInfo.SendAddonMessage("CPREM-RC", "-", strupper(ClassicPremade_GroupType()));
    ClassicPremadeReadyButton:SetEnabled(false);
  end
end)
